import {getGuid, isEmptyObject, isObj, serialize, getOfflineTrans} from './util'

let commonFetch = (url, params, successCallBack, failureCallBack, analysisData) => {
  
  const traceId = getGuid()
  
  let defaultHeaders = {
    'Accept': 'application/json, application/xml, text/play, text/html, *.*',
    'Content-Type': 'application/json; charset=utf-8'
  }
  if (analysisData) {
    const { loginUserInfo = {Id: 0}, tenantInfo = {Id: 0}} = BSGlobal || {}
    const uid = loginUserInfo.Id && typeof (loginUserInfo.Id) !== 'string' ? loginUserInfo.Id : 0;
    const tid = tenantInfo.Id &&  typeof (tenantInfo.Id) !== 'string' ? tenantInfo.Id : 0;
    const { 
      sourceActionName: fal = '',  // 中文描述即来源动作名称
      sourceActionCode: fan = '', // 按钮编码
      pages,     // 页面名称
      pageCode: fpn = ''    // 页面编码
    } = analysisData || {};

    defaultHeaders = Object.assign({}, defaultHeaders, {
      'ftc': traceId,
      fal: encodeURIComponent(fal),
      fpl: encodeURIComponent(pages),
      fan,
      fpn,
      'user_id': uid,
      'tenant_id': tid,
      fapp: BSGlobal && BSGlobal.application || '',
      fver: BSGlobal && BSGlobal.appVersion || ''
    })
  }
  const defaultParams = {
    headers: defaultHeaders,
    method: 'GET',
    credentials: 'include'
  }
  params = Object.assign({}, defaultParams, params)
  params['headers'][ 'EagleEye-TraceID'] =  traceId
  params['headers'][ 'X-Sourced-By'] =  'ajax'

  const startTime = new Date()

  if (window.customParam && isObj(window.customParam) && !isEmptyObject(window.customParam)) {
    var joinParam = url.indexOf("?") != -1 ? '&' : '?';
    url = url + joinParam + serialize(window.customParam);
  } 

  fetch(url, params).then((res) => {
    if (!res.ok) {
      res.json = function() {
        return this.text().then(function(data) {
          try {
            return JSON.parse(data);
          } catch(e) {
            return Promise.reject(res);
          }
        })
      }
      res.json().then(function(data){
          failureCallBack(data)
      })
      return;
    }

    res.json().then((data) => {
      const endTime = new Date()
      successCallBack(data)
      const { method = 'GET' } = params
      window.iTalentPMS && window.iTalentPMS._getAjaxTiming && window.iTalentPMS._getAjaxTiming(url, method, startTime, endTime, traceId, analysisData)
    })
  }).catch((error)=> {
    if (navigator && !navigator.onLine) {
      error = {
        message: getOfflineTrans(),
        isOffLine: true
      }
    }
    failureCallBack(error)
  })
}

export default commonFetch