import { createStore, applyMiddleware } from 'redux'
import { shallow, mount, render } from 'enzyme';
import * as reducer from '../testRely/reducer'
import { thunk} from '../testRely/middleware'

describe('applyMiddleware',function(){
	it('wraps dispath method with middleware once',function(){
		function test(spyMethods){
			return methods =>{
				spyMethods(methods)
				return next =>action =>next(action)
			}
		}
		const spy = jasmine.createSpy();
		const store = applyMiddleware(test(spy),thunk)(createStore)(reducer.todos)
		store.dispatch(reducer.addToDo('one'))
		store.dispatch(reducer.addToDo('two'))
		store.dispatch(reducer.addToDo('third'))

		expect(spy.calls.count()).toEqual(1);
		expect(spy.calls.allArgs()[0][0].getState).toEqual(jasmine.any(Function));
		expect(spy.calls.allArgs()[0][0].dispatch).toEqual(jasmine.any(Function));
		expect(store.getState()).toEqual([{id:1,text:'one'},{id : 2,text : 'two'},{id:3,text:'third'}]);
	})
	it('passes recursive dispatches through the middleware chain',function(){
		function test (spyMethods){
			return () =>next=>action =>{
				spyMethods(action)
				return next(action)
			}
		}
		const spy = jasmine.createSpy();
		const store = applyMiddleware(test(spy),thunk)(createStore)(reducer.todos)
		
		//为什么执行两次
		/*return store.dispatch(reducer.addToDoAsync('one')).then(()=>{
			expect(spy.calls.count()).toEqual(2)
		});*/
	})

	it('works with thunk middleware',function(){
		const store = applyMiddleware(thunk)(createStore)(reducer.todos)
		store.dispatch(reducer.addToDoIfEmpty('Hello'))
		expect(store.getState()).toEqual([{
			id : 1,
			text : 'Hello'
		}]);
		store.dispatch(reducer.addToDo('Word'))
		expect(store.getState()).toEqual([{
			id : 1,
			text : 'Hello'
		},{
			id : 2,
			text : 'Word'
		}])

		/*store.dispatch(reducer.addToDoAsync('Three')).then(()=>{
			expect(store.getState()).toEqual([{
				id : 1,
				text : 'Hello'
			},{
				id : 2,
				text : 'Word'
			},{
				id : 3,
				text : 'Three'
			}])
			done()
		});*/
	})

})