import { bindActionCreators,createStore } from 'redux'
import * as actionCreators from '../testRely/reducer'
import { shallow, mount, render } from 'enzyme';

describe('bindActionCreators',function(){
	
	let store
	let actionCreateorFunctions

	beforeEach(function(){
		
		store = createStore(actionCreators.todos)
		
		actionCreateorFunctions = {...actionCreators}
		
		Object.keys(actionCreateorFunctions).forEach(key=>{
			
			if(typeof actionCreateorFunctions[key] !='function'){
				delete actionCreateorFunctions[key]
			}
		})
	});

	it('wraps the action creators with the dispatch function',function(){
		
		const boundActionCreateors = bindActionCreators(actionCreators,store.dispatch)

		expect(Object.keys(boundActionCreateors)).toEqual(Object.keys(actionCreateorFunctions))

		const action = boundActionCreateors.addToDo('one');
		expect(action).toEqual(actionCreators.addToDo('one'))
		expect(store.getState()).toEqual([{
			id : 1,
			text : 'one'
		}])
	})

	/*it('skips non-function values in the passed Object',function(){

		const boundActionCreateors = bindActionCreators({
			...actionCreators,
			foo : 42
		},store.dispatch)

		expect(Object.keys(boundActionCreateors)).toEqual(Object.keys(actionCreateorFunctions))
	})*/

	it('supports wrapping a single funciton only',function(){
		const actionCreator = actionCreators.addToDo
		const boundActionCreateor = bindActionCreators(actionCreator,store.dispatch)

		const action = boundActionCreateor('Hello')
		expect(action).toEqual(actionCreator('Hello'))
		expect(store.getState()).toEqual([{
			id : 1,
			text : 'Hello'
		}])
	})

	it('throws for an undefined actionCreator',function(){
		expect(()=>{
			bindActionCreators(undefined,store.dispatch)
		}).toThrow()
	})

	it('throws for a null actionCreator',function(){
		expect(()=>{
			bindActionCreators(null,store.dispatch)
		}).toThrow()
	})

	it('throws for a primitive actionCreator',function(){
		expect(()=>{
			bindActionCreators('string',store.dispatch)
		}).toThrow()
	})
}); 