import React from 'react';
import { shallow, mount, render } from 'enzyme'; 

//shallow 就是官方的shallow rendering 封装
//render 将react 组件渲染成静态的HTML字符串,然后分析这段HTML代码的结构，返回一个对象
//mount 方法用于将React 组件加载为真实的DOM节点

describe('test React Children', function() {

  it('React.Children', function() {
      const SomeComponent = React.createClass({
          render(){
              return (
                  <div>
                      <span className="child" />
                      <span className="child" />
                  </div>
              );
          }
      });
      const someComponent = shallow(<SomeComponent />);
      expect(someComponent.find('div').children().length).toEqual(2);
  })

  it('React.Children.map', function() {
      const ChildrenComponent = React.createClass({
          render(){
              return (
                  <div>
                      <div className="foo">bax</div>
                      <div className="foo">bar</div>
                      <div className="foo">baz</div>
                  </div>
              );
          }
      });
      
      const childrenComponent = shallow(<ChildrenComponent />);
      const texts = childrenComponent.find('.foo').map(node => node.text());
      expect(texts).toEqual([ 'bax', 'bar', 'baz' ]);
    });

    it('React.Children.forEach', function() {
      const ChildComponent = React.createClass({
          render(){

              return (
                  <div>
                     <div className="foo bax" />
                     <div className="foo bar" />
                     <div className="foo baz" />
                  </div>
              );
          }
      });
      
      
      const childComponent = shallow(<ChildComponent/>);
      childComponent.find('.foo').forEach(function (node) {
          expect(node.hasClass('foo')).toEqual(true);
      });
  });

  it('React.Children.count', function() {
    var child = React.createClass({
        render (){
            return (<div>test</div>);
        }
    });
    const OneComponent = React.createClass({ 
        render(){
            return (
                <div>
                    {this.props.datas.map(child =>
                        <child key={child.id} value={child.name}/>
                    )}
                </div>
            );
        }
    });
    const datas = [{name: 'test1', id: 1}, {name: 'test2', id: 2}, {name: 'test3', id: 3}];
    const oneComponent = shallow(<OneComponent datas={datas}/>);
    var count = React.Children.count(oneComponent.props().children);
    expect(count).toEqual(datas.length);
          
  });
  it('React.Children.only', function() {
    const OnlyComponent = React.createClass({
        render(){
            return (
                <div>
                    <span>{this.props.name}</span>
                </div>
            );
        }
    });
    var name = 'test';
    const onlyComponent = shallow(<OnlyComponent name={name}/>);
    var child = React.Children.only(onlyComponent.props().children);
    expect(child.props.children).toEqual(name);
  });
})
describe('test ReactComponent', function() {
  it('setState', function() {
      var SomeComponent = React.createClass({
          getInitialState(){
              return {
                  clicked: false
              }
          },
          handelClick(){
              this.setState({clicked: true});
          },
          render(){
              return (<input type="checkbox" ref="checkbox" defaultChecked={this.state.clicked} onClick={this.handelClick} />);
          }
      });
      var someComponent = mount(<SomeComponent/>);
      someComponent.find('input[type="checkbox"]').simulate('click');
      expect(true).toEqual(someComponent.state().clicked);
  });
})
describe('can pass react attrs when rendering',function(){

	it('can pass proptypes when rendering', function() {
    //propTypes
    var PropTypesComponent = React.createClass({
            propTypes : {
                optionalArray: React.PropTypes.array.isRequired,
                optionalBool: React.PropTypes.bool.isRequired,
                optionalFunc: React.PropTypes.func.isRequired,
                optionalNumber: React.PropTypes.number.isRequired,
                optionalObject: React.PropTypes.object.isRequired,
                optionalString: React.PropTypes.string.isRequired
            },
            render: function() {
                return (<div></div>);
            }
        });

	   const optionalString = 'test';
	   const optionalArray = [1,2];
	   const optionalBool = true;
	   const optionalFunc = function(){};
	   const optionalNumber = 1;
	   const optionalObject = {};

	   const propTypesComponent =  mount(<PropTypesComponent optionalString={optionalString}  optionalArray={optionalArray} optionalBool={optionalBool} optionalFunc={optionalFunc} optionalNumber={optionalNumber} optionalObject={optionalObject}/>);
	    
	    expect(optionalString).toEqual(propTypesComponent.props().optionalString);
	    expect(optionalArray).toEqual(propTypesComponent.props().optionalArray);
	    expect(optionalBool).toEqual(propTypesComponent.props().optionalBool);
	    expect(optionalFunc).toEqual(propTypesComponent.props().optionalFunc);
	    expect(optionalNumber).toEqual(propTypesComponent.props().optionalNumber);
	    expect(optionalObject).toEqual(propTypesComponent.props().optionalObject);
  	
    });

  	it('can pass getInitialState when rendering',function(){
        //getInitialState 初始化state
  		var SomeComponent   = React.createClass({
			getInitialState (){
				return {clicked: false};
			},
			render () {
				return (<div></div>);
			}
		});
  		var result = mount(<SomeComponent />);
  		expect(false).toBe(result.state().clicked);
  	});

  	it('can pass getDefaultProps when rendering',function(){
        //getDefaultProps 默认属性
  		var SomeComponent   = React.createClass({
			getDefaultProps (){
				return {clicked: false};
			},
			render () {
				return (<div></div>);
			}
		});
		var obj = {
			className : 'test',
			clicked : false
		};
  		var result = mount(<SomeComponent className={'test'}/>);
  		expect(result.props()).toEqual(obj);
  	});

  	/*it('can pass mixins when rendering',function(){
        //mixins 
  		var SetIntervalMixin = {
  			componentWillMount : function(){
  				this.intervals = [];
  			},
  			setMixin : function(){
  				this.intervals.push(setInterval.apply(null,arguments));
  			},
  			componentWillUnmount : function(){
  				this.intervals.map(clearInterval);
  			}
  		};
  		var TickTock = React.createClass({
  			mixins : [SetIntervalMixin],
  			getInitialState: function() {
    			return {seconds: 0};
  			},
  			componentDidMount : function(){
  				this.setMixin(this.tick, 1000);
  			},
  			tick : function(){
  				this.setState({seconds: this.state.seconds + 1});
  			},
  			render : function (){
  				return (
  					<p>{this.state.seconds}</p>
  				);
  			}
  		});
  		var result = mount(<TickTock />);
  		setTimeout(function(){
  			expect(result.state().seconds).toEqual(1);
  		},1000);
  		
  	});*/

  	it('it can pass statics when rendering',function(){
        //statics 静态方法
  		var MyComponent = React.createClass({
  			statics : {
  				customMethod : function(foo){
  					return foo =='bar';
  				}
  			},
  			render : function(){

  			}
  		});
  		expect(MyComponent.customMethod('bar')).toBe(true);
  	});

  	it('it can pass componentWillMount  when rendering',function(){
  		//在初始化渲染执行之前立刻调用，紧执行一次
  		var Component = React.createClass({
  			
  			getInitialState : function(){
  				return {'name' : 'test'};
  			},
  			componentWillMount : function(){
  				this.setState({'name':'test1'});
  			},

  			render : function(){
  				return (<div></div>);
  			}
  		});
  		var result = mount(<Component />);
  		expect(result.state().name).toEqual('test1');
  	});
});