/**
 * Created by liubo on 15/8/12.
 */
var webpack = require("webpack");
var	path = require("path");
var root = path.join(__dirname, "./"); //根目录
var node_modules = path.join(root,'./node_modules');

var ExtractTextPlugin = require("extract-text-webpack-plugin");
var CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin');
var VersionPlugin = require('@beisen/versionplugin');

//执行脚本自动打tag,若想升大版本或打测试版本，需手动填入newTag的字符串
var newTag = '' || require('./getLastestTag.js').makeNewTag();

module.exports = function(options) {

  //入口
  var entry = options.entry;

  //压缩css
  var ExtractTextPluginConfig;
  var ExtractTextPluginConfig1; //组件库用到的无版本css
  if(options.minimize){
    ExtractTextPluginConfig = new ExtractTextPlugin("css/[name]-[hash].min.css",{disable: false,allChunks: true});
    ExtractTextPluginConfig1 = new ExtractTextPlugin("css/[name].min.css",{disable: false,allChunks: true});
  }else{
    ExtractTextPluginConfig = new ExtractTextPlugin("css/[name].css",{disable: false,allChunks: true});
  }
  
  var output = {
      library: "__talent-libs__",
      libraryTarget: "umd",
      path: path.resolve(root, "./dist/"),
      filename: (options.minimize)? "[name]-" + newTag + ".min.js" : "[name].js",
      sourceMapFilename: (options.minimize)? "[name]-" + newTag + ".min.js.map": "[name].js.map"
  };

  //加载器
  var loaders = [
      {
        test: /\.js[x]?$/
        ,exclude: /node_modules/
        ,loader: 'babel'
      },
      {
        test: /\.css$/, loader: ExtractTextPlugin.extract(
        "style-loader",
        "css-loader?sourceMap",
        {
            publicPath: "../"
        }
      )}, 
      {
        test: /\.scss$/, loader: ExtractTextPlugin.extract(
        "style-loader",
        "css-loader?sourceMap!sass-loader",
        {
            publicPath: "../"
        }
      )}, 
      {
        test: /\.(ttf|eot|svg|woff(2)?)(\?[^('|")]*)?$/, 
        loader: "file-loader?name=images/[hash:8].[name].[ext]"
      }, 
      {
        test: /\.(png|jpg|gif)$/,
        loader: "file-loader?name=images/[hash:8].[name].[ext]"
      }
  ];

  var DefinePluginConfig = !options.minimize? {
      __CLIENT__: true,
      __SERVER__: false,
      __DEVELOPMENT__: true,
      __DEVTOOLS__: false
  } : {
      __DEVELOPMENT__: false,
      'process.env': {
          // Useful to reduce the size of client-side libraries, e.g. react
          NODE_ENV: JSON.stringify('production')
      }
  }

  //插件
  var plugins = [
    new webpack.optimize.OccurenceOrderPlugin(),
    new webpack.optimize.DedupePlugin(),
    new webpack.DefinePlugin(DefinePluginConfig),
    ExtractTextPluginConfig,
    new VersionPlugin()
  ];

  if (options.minimize){
    plugins.push(
      new webpack.optimize.UglifyJsPlugin({
        compress: {
            warnings: false
        }
        ,sourceMap: true
        ,mangle: {
            except: ['$', 'm', 'webpackJsonpCallback']
        }
      }),
      ExtractTextPluginConfig1
    );
    loaders.unshift({ test: /\.js$/, loader: 'strip-debug' })
  }

   // 使用webpack优化工具
  var WebapckComiplePlugin = require('@beisen/webpack-compile-plugin');
  plugins.push(new WebapckComiplePlugin({ 'singler': 'TALENT_CORE' }));

  return {
    entry: entry
    ,output: output
    ,module: {
        loaders: loaders
        ,preLoaders: [
            { test: /\.js$/, loader: "source-map" }
        ]
    }
    ,plugins: plugins
    ,resolve: {
        extensions: ['', '.json', '.webpack.js', '.web.js', '.js', '.jsx']
    }
    ,externals: options.externals || {}
    ,devtool: options.devtool
  }
}
