/**
 * Created by lhj on 16/3/29.
 */
var webpack = require('webpack');
var path = require('path');
var hotFixPlugin = require('@beisen/hotfixplugin');
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin');
var hotfixCSS = [];
var hotfixFile = [];

//@appName 项目名称
//@root  当前程序根路径
//@entry 程序入口
//@devServer 启动开发服务器
//@hot 启动hot服务器
//@devtool 开发工具
//@minimize 启用压缩
//@debug  启用debug
//@dirName 输出目录
//@host 主机名
//@port 端口


module.exports = function(options) {

  //路径
  var appRoot = path.join(options.root, './app');
  var bowerRoot = path.join(options.root, './app/scripts/vendor');
  var nodeRoot = path.join(options.root, './node_modules');

  var Filename = "[name].bundle.js";
  //加载器
  var defaultLoaders = [
    {
        test: /\.json$/, loader: 'json-loader',
        exclude: path.join(options.root, 'node_modules')
    }
  ]
  var loaders = require("../loaders-dev")(options).concat(options.loaders || defaultLoaders);
  var externals = require("../externals");

  var changeSet = options.changeSet;


	for(var j = 0, length = changeSet.length; j < length; j ++){
		var tempFileName = changeSet[j];
		if(tempFileName.indexOf(".scss")!=-1){
			hotfixCSS.push("./"+tempFileName)
		}else{
			hotfixFile.push(tempFileName)
		}
	} 
  
  var output = {
    path: path.join(options.root, "./dist"),
    libraryTarget: 'umd',
    filename: Filename,
    chunkFilename: "[name].chunk.js"
  }

  if (options.crossOriginLoading) {
    output.crossOriginLoading = options.crossOriginLoading
  }
  
  //别名
  var alias = {
      "bowerRoot": bowerRoot
      , "nodeRoot": nodeRoot
  };

  //开发服务器
  var devServer = {
      contentBase: path.join(options.root, "./dist")
      ,color: true
      ,hot: options.hot
      ,inline: true
      ,progress: true
      ,host: options.host
      ,port: options.port
  }

  //压缩css
  var ExtractTextPluginConfig = new ExtractTextPlugin("css/[name].css",{disable: false,allChunks: true});

  //插件
  var plugins = [
      new webpack.ProvidePlugin({
          "React": "react"
      }),
      //定义特殊标签域的处理方式
      new webpack.DefinePlugin({
          __DEV__: JSON.stringify(JSON.parse(true || 'false'))
      })
      //优化-减少重复的输出
      , ExtractTextPluginConfig
      , new webpack.optimize.CommonsChunkPlugin({
          name: "common",

          // filename: "vendor.js"
          // (Give the chunk a different name)

          minChunks: Infinity
          // (with more entries, this ensures that no other module
          //  goes into the vendor chunk)
      })
      ,new CaseSensitivePathsPlugin()
  ];

  var hotfixP = [
  	 new webpack.ProvidePlugin({
          "React": "react"
      }),
      //定义特殊标签域的处理方式
      new webpack.DefinePlugin({
          __DEV__: JSON.stringify(JSON.parse(true || 'false'))
      })
      //优化-减少重复的输出
      , ExtractTextPluginConfig
      , new webpack.optimize.CommonsChunkPlugin({
          name: "common",

          // filename: "vendor.js"
          // (Give the chunk a different name)

          minChunks: Infinity
          // (with more entries, this ensures that no other module
          //  goes into the vendor chunk)
      })
      ,new CaseSensitivePathsPlugin()
  ]

   var hotFixPluginConfig =  {
          mapPath: path.join(options.root, "./dist/mapFile")
          ,mapFile:"/mapFile.js"
          ,mapModulePath: path.join(options.root, "./dist/mapModuleFile")
          ,mapModuleFile:"/mapFile.js"
          ,filter:"src"
  }

  // if(options.env == 'hotfix') {
    // var changeSet = require(path.join(options.root, './dist/changeSet'));
    hotFixPluginConfig['target'] = 'hotfix';
    hotFixPluginConfig['hotfixFile'] = hotfixFile;
  // }

  var hotfix = new hotFixPlugin(hotFixPluginConfig);

  hotfixP.push(hotfix);
  //生成承载页
  var HtmlWebpackPlugin = require('html-webpack-plugin');

  var HTMLWebpackPluginConfig = new HtmlWebpackPlugin({
      template: options.root + '/server/index.html',
      hash: true,
      filename: 'index.html',
      inject: 'body'
  });

  // plugins.push(HTMLWebpackPluginConfig);

  //复制libs
  var CopyWebpackPlugin = require('copy-webpack-plugin');
  var CopyWebpackPluginConfig = new CopyWebpackPlugin([
      { from: nodeRoot + '/@beisen/talent-core/dist/', to: options.root},
      { from: nodeRoot + '/@beisen/beisencloud-mobile/dist/', to: options.root}
  ]);

  // plugins.push(CopyWebpackPluginConfig);
  
  //热加载插件
  // if(options.hot){
  //   var HotReloader = new webpack.HotModuleReplacementPlugin();
  //   plugins.push(HotReloader);
  // }
  var returnArray = [];
  if(hotfixCSS.length > 0){
  	returnArray .push({
  	entry:{
  		hotfix:hotfixCSS
  	}
  	,output: output
  	,resolve: {
        //查找module的话从这里开始查找
        //root: [appRoot, nodeRoot, bowerRoot] //设置绝对路径
        //模块别名定义，方便后续直接引用别名，无须多写长长的地址
        alias: alias
        //自动扩展文件后缀名，意味着我们require模块可以省略不写后缀名
        ,extensions: ['', '.js', '.jsx', '.css']
        ,modulesDirectories: options.dir 
    }
     //开发服务器配置
    ,devServer: devServer
    //开发工具
    ,devtool: options.devtool
    //模块
    ,module: {
        //加载器
        loaders: loaders
    }
    //插件项
    ,plugins: plugins
    //外部代码库
    ,externals: externals
    ,debug: options.debug
  })
  }
  if(hotfixFile.length > 0){
  	returnArray.push({entry:options.entry
  	,output: output
  	,resolve: {
        //查找module的话从这里开始查找
        //root: [appRoot, nodeRoot, bowerRoot] //设置绝对路径
        //模块别名定义，方便后续直接引用别名，无须多写长长的地址
        alias: alias
        //自动扩展文件后缀名，意味着我们require模块可以省略不写后缀名
        ,extensions: ['', '.js', '.jsx', '.css', '.json']
        ,modulesDirectories: options.dir 
    }
     //开发服务器配置
    ,devServer: devServer
    //开发工具
    ,devtool: options.devtool
    //模块
    ,module: {
        //加载器
        loaders: loaders
    }
    //插件项
    ,plugins: hotfixP
    //外部代码库
    ,externals: externals
    ,debug: options.debug
  })
  }

  return returnArray;
  
};