/**
 * Created by lhj on 16/3/29.
 */
var webpack = require('webpack');
var path = require('path');
var hotFixPlugin = require('@beisen/hotfixplugin');
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var makeFileInEntry = require("../makeFileInEntry");

//@appName 项目名称
//@root  当前程序根路径
//@entry 程序入口
//@devServer 启动开发服务器
//@hot 启动hot服务器
//@devtool 开发工具
//@minimize 启用压缩
//@debug  启用debug
//@dirName 输出目录
//@host 主机名
//@port 端口
module.exports = function(options) {

  //路径
  var appRoot = path.join(options.root, './app');
  var bowerRoot = path.join(options.root, './app/scripts/vendor');
  var nodeRoot = path.join(options.root, './node_modules');

  var Filename = "[name]-[chunkhash].bundle.min.js";

  //加载器
  var defaultLoaders = [
    {
        test: /\.json$/, loader: 'json-loader',
        exclude: path.join(options.root, 'node_modules')
    }
  ]
  var loaders = require("../loaders")(options).concat(options.loaders || defaultLoaders);
  var externals = require("../externals")

  var output = {
    path: path.join(options.root, "./dist"),
    libraryTarget: 'umd',
    filename: Filename,
    chunkFilename: "[name]-[chunkhash].chunk.min.js"
  }

  if (options.crossOriginLoading) {
    output.crossOriginLoading = options.crossOriginLoading
  }
  
  //别名
  var alias = {
      "bowerRoot": bowerRoot
      , "nodeRoot": nodeRoot
  };

  //压缩css
  var ExtractTextPluginConfig = new ExtractTextPlugin("css/[name]-[hash].min.css",{disable: false,allChunks: true});

  //公共模块合并
  var commonfile = options.commonfile;
  if(commonfile){
      commonfile = path.join(options.root, commonfile);
      var entry = options.entry;
      var entryPath = path.join(options.root, entry.main);
      var endPath = path.join(options.root, './src/newMain.js');
      makeFileInEntry(commonfile, entryPath, endPath);
      entry.main = './src/newMain.js';
  }
  //hotfix
  var hotFixPluginConfig =  {
          mapPath: path.join(options.root, "./dist/mapFile")
          ,mapFile:"/mapFile.js"
          ,mapModulePath: path.join(options.root, "./dist/mapModuleFile")
          ,mapModuleFile:"/mapFile.js"
          ,filter:"src"
  }

  if(options.env == 'hotfix') {
    var changeSet = require(path.join(options.root, './dist/changeSet'));
    hotFixPluginConfig['target'] = 'hotfix';
    hotFixPluginConfig['hotfixFile'] = changeSet;
  }
  //插件
  var plugins = [
      new webpack.ProvidePlugin({
          "React": "react"
      })
      //,
      //定义特殊标签域的处理方式
      // new webpack.DefinePlugin({
      //     __DEV__: JSON.stringify(JSON.parse(true || 'false'))
      // })
      , ExtractTextPluginConfig
      , new webpack.optimize.CommonsChunkPlugin({
          name: "common",

          // filename: "vendor.js"
          // (Give the chunk a different name)

          minChunks: Infinity
          // (with more entries, this ensures that no other module
          //  goes into the vendor chunk)
      })
      ,new hotFixPlugin(hotFixPluginConfig)
  ];

  const exec = require('child_process').exec;
  exec("cp " + nodeRoot + '/@beisen/talent-core/dist/talent-libs-*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/chaos-ui/dist/ChaosUI-*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/chaos-ui/dist/1.*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/beisencloud-mobile/dist/BeisenCloudMobile-*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/beisencloud-mobile/dist/1.*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/beisencloud-mobile/dist/2.*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });
  exec("cp " + nodeRoot + '/@beisen/beisencloud-mobile/dist/3.*' + ' ' + options.root + 'dist/', (error, stdout, stderr) => {
    if (error) {
      return;
    }
  });

  plugins.push(
    new webpack.DefinePlugin({
        __DEBUG__: true  //屏蔽console调试代码
        ,"process.env": {
            NODE_ENV: JSON.stringify("production")
        }
    }),
    new webpack.optimize.UglifyJsPlugin({
        compress: {
            warnings: false
        }
        ,sourceMap: true
        ,mangle: {
            except: ['FormElement', 'NavItem']
        }
        ,minimize: true
    }),
    new webpack.NoErrorsPlugin()
  );

  var WebapckComiplePlugin = require('@beisen/webpack-compile-plugin');
  plugins.push(new WebapckComiplePlugin({ 'singler': options.singler }));

  return{
    //context: path.resolve(options.root, "app")
    //应用程序入口
    entry: options.entry
    //输出项配置
    ,output: output
    //其它解决方法配置
    ,resolve: {
        //查找module的话从这里开始查找
        //root: [appRoot, nodeRoot, bowerRoot] //设置绝对路径
        //模块别名定义，方便后续直接引用别名，无须多写长长的地址
        alias: alias
        //自动扩展文件后缀名，意味着我们require模块可以省略不写后缀名
        ,extensions: ['', '.js', '.jsx', '.css', '.json']
        ,modulesDirectories: options.dir 
    }
    //开发工具
    ,devtool: options.devtool
    //模块
    ,module: {
        //加载器
        loaders: loaders
    }
    //插件项
    ,plugins: plugins
    //外部代码库
    ,externals: externals
    ,debug: options.debug
  }
};